==> varsnap-1.3.1.tgz: package/varsnap/core.ts <==
'use strict';

import deepEqual = require('fast-deep-equal');
import querystring = require('querystring');
import xhr = require('request');

const produceURL = 'https://www.varsnap.com/api/snap/produce/';
const consumeURL = 'https://www.varsnap.com/api/snap/consume/';
const produceTrialURL = 'https://www.varsnap.com/api/trial/produce/';
export const version = 'v1.3.1';

export const configKeys = {
  varsnap: 'varsnap',
  env: 'env',
  producerToken: 'producerToken',
  consumerToken: 'consumerToken',
  logger: undefined,
};
let Producers: Producer[] = [];
let Consumers: Consumer[] = [];

export interface Logger {
  debug(s: string): void;
  log(s: string): void;
  error(s: string): void;
}

export class Util {
  public static logger: Logger = console; // eslint-disable-line no-console

  static getConfig(config, configKey): string {
    if (!config) {
      return '';
    }
    let val = config[configKey];
    if (!val) {
      return '';
    }
    val = val.toString().toLowerCase();
    return val;
  }

  static getSignature(targetFunc): string {
    let signature = targetFunc.toString().split('{')[0];
    signature = signature.replace(/^\s+|\s+$/g, '');
    return 'javascript.' + version + '.' + signature;
  }

  static ajax(url, requestData) {
    const options = {
      method: 'POST',
      uri: url,
      body: querystring.stringify(requestData),
      headers: {'Content-Type': 'application/x-www-form-urlencoded'},
    };
    const ajaxPromise = new Promise((resolve, reject) => {
      xhr(options, (err, resp, body) => {
        if (err) {
          return reject(err);
        }
        const responseCode = Math.round(resp.statusCode / 100);
        if (responseCode != 2) {
          return reject(new Error('Status: ' + resp.statusCode));
        }
        body = JSON.parse(body);
        return resolve(body);
      });
    });
    return ajaxPromise;
  }

  static debug(data) {
    Util.logger.debug(data);
  }

  static log(data) {
    Util.logger.log(data);
  }

  static error(data) {
    Util.logger.error(data);
  }

  static resetProducers() {
    Producers = [];
  }

  static resetConsumers() {
    Consumers = [];
  }

  static getProducers() {
    return Producers;
  }

  static getConsumers() {
    return Consumers;
  }

  static limitString(x) {
    x = String(x);
    const limit = 30;
    const ellipsis = '...';
    if (x.length <= limit) {
      return x;
    }
    return x.substring(0, 30 - ellipsis.length) + ellipsis;
  }
}

export class Producer {
  public targetFunc;
  public config;

  constructor(targetFunc, config) {
    this.targetFunc = targetFunc;
    this.config = config;
  }

  isEnabled() {
    if (Util.getConfig(this.config, configKeys.varsnap) !== 'true') {
      return false;
    }
    if (Util.getConfig(this.config, configKeys.env) !== 'production') {
      return false;
    }
    if (!Util.getConfig(this.config, configKeys.producerToken)) {
      return false;
    }
    return true;
  }

  static serialize(data) {
    return JSON.stringify(data);
  }

  produce(args, output) {
    if (!this.isEnabled()) {
      return;
    }
    Util.debug('Sending call to Varsnap');
    const data = {
      producer_token: Util.getConfig(this.config, configKeys.producerToken), // eslint-disable-line @typescript-eslint/camelcase
      signature: Util.getSignature(this.targetFunc),
      inputs: Producer.serialize(args),
      prod_outputs: Producer.serialize(output), // eslint-disable-line @typescript-eslint/camelcase
    };
    Util.ajax(produceURL, data).catch((err) => {
      if(err.statusCode === 404) {
        Util.error('Cannot produce to Varsnap; check your producer token');
      } else {
        Util.error(err.error);
      }
    });
  }
}

export class Consumer {
  public targetFunc;
  public config;

  constructor(targetFunc, config) {
    this.targetFunc = targetFunc;
    this.config = config;
  }

  isEnabled() {
    if (Util.getConfig(this.config, configKeys.varsnap) !== 'true') {
      return false;
    }
    if (Util.getConfig(this.config, configKeys.env) !== 'development') {
      return false;
    }
    if (!Util.getConfig(this.config, configKeys.consumerToken)) {
      return false;
    }
    return true;
  }

  static deserialize(data) {
    if (data === '') {
      return undefined;
    }
    return JSON.parse(data);
  }

  consumeOnce() {
    if (!this.isEnabled()) {
      return;
    }
    const data = {
      consumer_token: Util.getConfig(this.config, configKeys.consumerToken), // eslint-disable-line @typescript-eslint/camelcase
      signature: Util.getSignature(this.targetFunc),
    };
    return Util.ajax(consumeURL, data).then((response: any) => {
      if (!response || response.status !== 'ok') {
        return false;
      }
      let allMatches = true;
      for(const snap of response.results) {
        const inputs = Consumer.deserialize(snap['inputs']);
        const prodOutputs = Consumer.deserialize(snap['prod_outputs']);
        let localOutputs = undefined;
        try {
          localOutputs = this.targetFunc(...inputs);
        }
        catch(err) {
          localOutputs = err;
        }
        const matches = deepEqual(prodOutputs, localOutputs);
        this.reportCentral(snap, localOutputs, matches).then((trialURL) => {
          this.reportLog(snap['id'], inputs, prodOutputs, localOutputs, matches, trialURL);
        });
        allMatches = allMatches && matches;
      }
      return allMatches;
    }).catch((error) => {
      Util.error('Cannot contact Varsnap: ' + error);
      return false;
    });
  }

  reportCentral(snap, localOutputs, matches) {
    const data = {
      'consumer_token': Util.getConfig(this.config, configKeys.consumerToken),
      'snap_id': snap.id,
      'test_outputs': Producer.serialize(localOutputs),
      'matches': matches,
    };
    return Util.ajax(produceTrialURL, data).then((response: any) => {
      if (!response || response.status !== 'ok') {
        return false;
      }
      const trialURL = response['trial_url'];
      return trialURL;
    }).catch((error) => {
      Util.error('Cannot upload test results to Varsnap: ' + error);
      return false;
    });
  }

  reportLog(snapID, inputs, prodOutputs, localOutputs, matches, trialURL) {
    const functionName = Util.getSignature(this.targetFunc);
    Util.log('Testing with Varsnap uuid:     ' + snapID);
    Util.log('Function:                      ' + functionName);
    Util.log('Function input args:           ' + Util.limitString(inputs));
    Util.log('Production function outputs:   ' + Util.limitString(prodOutputs));
    Util.log('Your function outputs:         ' + Util.limitString(localOutputs));
    Util.log('Matching outputs:              ' + matches);
    Util.log('Trial URL:                     ' + trialURL);
    Util.log('');
  }
}

export function varsnap(func) {
  if (varsnap.config === undefined) {
    console.error('Varsnap has no config; disabling.');
    return func;
  }
  if (varsnap.config.logger) {
    Util.logger = varsnap.config.logger;
  }
  const producer = new Producer(func, varsnap.config);
  const consumer = new Consumer(func, varsnap.config);

  const wrapped = function(...args) {
    let result = '';
    try {
      result = func(...args);
    }
    catch(err) {
      producer.produce(args, err);
      throw(err);
    }
    producer.produce(args, result);
    return result;
  };

  wrapped.producer = producer;
  wrapped.consumer = consumer;

  Producers.push(producer);
  Consumers.push(consumer);
  return wrapped;
}
varsnap.config = configKeys;
varsnap.version = version;

export async function runTests() {
  const consumers = Util.getConsumers();
  let status = true;
  for(const consumer of consumers) {
    const testStatus = await consumer.consumeOnce();
    status = status && testStatus;
  }
  return status;
}
varsnap.runTests = runTests;

==> varsnap-1.3.2.tgz: package/varsnap/core.ts <==
import deepEqual = require('fast-deep-equal');
import querystring = require('querystring');
import xhr = require('request');

const produceURL = 'https://www.varsnap.com/api/snap/produce/';
const consumeURL = 'https://www.varsnap.com/api/snap/consume/';
const produceTrialURL = 'https://www.varsnap.com/api/trial/produce/';
export const version = 'v1.3.2';

export interface ConfigKeys {
  varsnap: string;
  env: string;
  producerToken: string;
  consumerToken: string;
  logger: Logger;
}

export const configKeys: ConfigKeys = {
  varsnap: 'varsnap',
  env: 'env',
  producerToken: 'producerToken',
  consumerToken: 'consumerToken',
  logger: undefined,
};
let Producers: Producer[] = [];
let Consumers: Consumer[] = [];

export interface Logger {
  debug(s: string): void;
  log(s: string): void;
  error(s: string): void;
}

export class Util {
  public static logger: Logger = console; // eslint-disable-line no-console

  static getConfig(config, configKey): string {
    if (!config) {
      return '';
    }
    let val = config[configKey];
    if (!val) {
      return '';
    }
    val = val.toString().toLowerCase();
    return val;
  }

  static getSignature(targetFunc: TargetFunction): string {
    let signature = targetFunc.toString().split('{')[0];
    signature = signature.replace(/^\s+|\s+$/g, '');
    return 'javascript.' + version + '.' + signature;
  }

  static ajax(url, requestData): Promise<object> {
    const options = {
      method: 'POST',
      uri: url,
      body: querystring.stringify(requestData),
      headers: {'Content-Type': 'application/x-www-form-urlencoded'},
    };
    const ajaxPromise = new Promise<object>((resolve, reject) => {
      xhr(options, (err, resp, body) => {
        if (err) {
          return reject(err);
        }
        const responseCode = Math.round(resp.statusCode / 100);
        if (responseCode != 2) {
          return reject(new Error('Status: ' + resp.statusCode));
        }
        body = JSON.parse(body);
        return resolve(body);
      });
    });
    return ajaxPromise;
  }

  static debug(data): void {
    Util.logger.debug(data);
  }

  static log(data): void {
    Util.logger.log(data);
  }

  static error(data): void {
    Util.logger.error(data);
  }

  static resetProducers(): void {
    Producers = [];
  }

  static resetConsumers(): void {
    Consumers = [];
  }

  static getProducers(): Producer[] {
    return Producers;
  }

  static getConsumers(): Consumer[] {
    return Consumers;
  }

  static limitString(x: TargetFunctionInput[] | TargetFunctionOutput): string {
    const limitedString = String(x);
    const limit = 30;
    const ellipsis = '...';
    if (limitedString.length <= limit) {
      return limitedString;
    }
    return limitedString.substring(0, 30 - ellipsis.length) + ellipsis;
  }
}

export class Producer {
  public targetFunc;
  public config;

  constructor(targetFunc, config) {
    this.targetFunc = targetFunc;
    this.config = config;
  }

  isEnabled(): boolean {
    if (Util.getConfig(this.config, configKeys.varsnap) !== 'true') {
      return false;
    }
    if (Util.getConfig(this.config, configKeys.env) !== 'production') {
      return false;
    }
    if (!Util.getConfig(this.config, configKeys.producerToken)) {
      return false;
    }
    return true;
  }

  static serialize(data: TargetFunctionInput[] | TargetFunctionOutput): string {
    return JSON.stringify(data);
  }

  produce(args: TargetFunctionInput[], output: TargetFunctionOutput): void {
    if (!this.isEnabled()) {
      return;
    }
    Util.debug('Sending call to Varsnap');
    const data = {
      producer_token: Util.getConfig(this.config, configKeys.producerToken), // eslint-disable-line @typescript-eslint/camelcase
      signature: Util.getSignature(this.targetFunc),
      inputs: Producer.serialize(args),
      prod_outputs: Producer.serialize(output), // eslint-disable-line @typescript-eslint/camelcase
    };
    Util.ajax(produceURL, data).catch((err) => {
      if(err.statusCode === 404) {
        Util.error('Cannot produce to Varsnap; check your producer token');
      } else {
        Util.error(err.error);
      }
    });
  }
}

interface ConsumeSnapResponse {
  status: string;
  results: Snap[];
}

interface Snap {
  id: string;
  inputs: string;
  prod_outputs: string;
}

interface ProduceTrialResponse {
  status: string;
  trial_url: string;
}

export class Consumer {
  public targetFunc;
  public config;

  constructor(targetFunc, config) {
    this.targetFunc = targetFunc;
    this.config = config;
  }

  isEnabled(): boolean {
    if (Util.getConfig(this.config, configKeys.varsnap) !== 'true') {
      return false;
    }
    if (Util.getConfig(this.config, configKeys.env) !== 'development') {
      return false;
    }
    if (!Util.getConfig(this.config, configKeys.consumerToken)) {
      return false;
    }
    return true;
  }

  static deserialize(data: string): TargetFunctionInput[] | TargetFunctionOutput {
    if (data === '') {
      return undefined;
    }
    return JSON.parse(data);
  }

  consumeOnce(): Promise<boolean> {
    if (!this.isEnabled()) {
      return;
    }
    const data = {
      consumer_token: Util.getConfig(this.config, configKeys.consumerToken), // eslint-disable-line @typescript-eslint/camelcase
      signature: Util.getSignature(this.targetFunc),
    };
    return Util.ajax(consumeURL, data).then((response: ConsumeSnapResponse) => {
      if (!response || response.status !== 'ok') {
        return false;
      }
      let allMatches = true;
      for(const snap of response.results) {
        const inputs: TargetFunctionInput[] = Consumer.deserialize(snap['inputs']) as TargetFunctionInput[];
        const prodOutputs: TargetFunctionOutput = Consumer.deserialize(snap['prod_outputs']);
        let localOutputs = undefined;
        try {
          localOutputs = this.targetFunc(...inputs);
        }
        catch(err) {
          localOutputs = err;
        }
        const matches = deepEqual(prodOutputs, localOutputs);
        this.reportCentral(snap, localOutputs, matches).then((trialURL) => {
          this.reportLog(snap['id'], inputs, prodOutputs, localOutputs, matches, trialURL);
        });
        allMatches = allMatches && matches;
      }
      return allMatches;
    }).catch((error) => {
      Util.error('Cannot contact Varsnap: ' + error);
      return false;
    });
  }

  reportCentral(snap: Snap, localOutputs: TargetFunctionOutput, matches: boolean): Promise<string|boolean> {
    const data = {
      'consumer_token': Util.getConfig(this.config, configKeys.consumerToken),
      'snap_id': snap.id,
      'test_outputs': Producer.serialize(localOutputs),
      'matches': matches,
    };
    return Util.ajax(produceTrialURL, data).then((response: ProduceTrialResponse) => {
      if (!response || response.status !== 'ok') {
        return false;
      }
      const trialURL = response['trial_url'];
      return trialURL;
    }).catch((error) => {
      Util.error('Cannot upload test results to Varsnap: ' + error);
      return false;
    });
  }

  reportLog(snapID: string, inputs: TargetFunctionInput[], prodOutputs: TargetFunctionOutput, localOutputs: TargetFunctionOutput, matches: boolean, trialURL: string|boolean): void {
    const functionName = Util.getSignature(this.targetFunc);
    Util.log('Testing with Varsnap uuid:     ' + snapID);
    Util.log('Function:                      ' + functionName);
    Util.log('Function input args:           ' + Util.limitString(inputs));
    Util.log('Production function outputs:   ' + Util.limitString(prodOutputs));
    Util.log('Your function outputs:         ' + Util.limitString(localOutputs));
    Util.log('Matching outputs:              ' + matches);
    let trialURLView = 'Cannot upload trial';
    if(trialURL) {
      trialURLView = String(trialURL);
    }
    Util.log('Trial URL:                     ' + trialURLView);
    Util.log('');
  }
}

export interface TargetFunctionInput {}  // eslint-disable-line @typescript-eslint/no-empty-interface
export interface TargetFunctionOutput {} // eslint-disable-line @typescript-eslint/no-empty-interface
export interface TargetFunction {
  (...args: TargetFunctionInput[]): TargetFunctionOutput;
}
export interface WrappedFunction {
  (...args: TargetFunctionInput[]): TargetFunctionOutput;
  producer: Producer;
  consumer: Consumer;
}

export function varsnap(func: TargetFunction): TargetFunction|WrappedFunction {
  if (varsnap.config === undefined) {
    console.error('Varsnap has no config; disabling.');
    return func;
  }
  if (varsnap.config.logger) {
    Util.logger = varsnap.config.logger;
  }
  const producer = new Producer(func, varsnap.config);
  const consumer = new Consumer(func, varsnap.config);

  const wrapped = function(...args: TargetFunctionInput[]): TargetFunctionOutput {
    let result: TargetFunctionOutput = undefined;
    try {
      result = func(...args);
    }
    catch(err) {
      producer.produce(args, err);
      throw(err);
    }
    producer.produce(args, result);
    return result;
  };

  wrapped.producer = producer;
  wrapped.consumer = consumer;

  Producers.push(producer);
  Consumers.push(consumer);
  return wrapped;
}
varsnap.config = configKeys;
varsnap.version = version;

export async function runTests(): Promise<boolean> {
  const consumers = Util.getConsumers();
  let status = true;
  for(const consumer of consumers) {
    const testStatus = await consumer.consumeOnce();
    status = status && testStatus;
  }
  return status;
}
varsnap.runTests = runTests;
